/*
 * Created by: 
 * Created: Saturday, March 24, 2007
 */

using System;
using System.Data;
using System.Data.SqlClient;
using Csla;
using Csla.Data;

namespace CogArtTech.Music.Library.Lists
{
    [Serializable]
    public class MusicianInstrumentsList : BusinessListBase<MusicianInstrumentsList, Instrument>
    {
        private static Musician _Musician;

        #region Business Methods

        public Instrument GetItem(int instrumentId)
        {
            foreach (Instrument instrument in this)
                if (instrument.Id == instrumentId)
                    return instrument;
            return null;
        }

        public void Unassign(Instrument instrument)
        {
            Remove(instrument.Id);
        }

        public void Unassign(int instrumentId)
        {
            Remove(instrumentId);
        }

        public void Assign(Instrument instrument)
        {
            if (!Contains(instrument))
            {
                foreach (Instrument item in InstrumentsList.GetList())
                {
                    if (item.Id == instrument.Id)
                    {
                        Instrument newInstrument = Instrument.NewInstrument();
                        newInstrument.Id = instrument.Id;
                        newInstrument.Description = instrument.Description;
                        this.Add(newInstrument);
                        break;
                    }
                }
            }
            else
                throw new InvalidOperationException(
                    string.Format("Instrument already assigned to {0} {1}", _Musician.FirstName, _Musician.LastName));
        }

        private new void Remove(Instrument instrument)
        {
            //Just hides the move method;
            base.Remove(instrument);
        }

        public bool Contains(int instrumentId)
        {
            foreach (Instrument instrument in this)
                if (instrument.Id == instrumentId)
                    return true;
            return false;
        }

        public bool ContainsDeleted(int instrumentId)
        {
            foreach (Instrument instrument in DeletedList)
                if (instrument.Id == instrumentId)
                    return true;
            return false;
        }

        internal void Update(Musician musician)
        {
            if (!this.IsDirty) return;

            this.RaiseListChangedEvents = false;
            foreach (Instrument obj in DeletedList)
            {
                RemoveInstrumentFromMusician(musician, obj.Id);
            }

            DeletedList.Clear(); // deletes taken care of by cascade

            // add/update any current child objects
            foreach (Instrument obj in this)
            {
                if (obj.IsNew)
                {
                    // Now add the instrument to the list
                    using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
                    {
                        cn.Open();
                        using (SqlCommand cmd = new SqlCommand("usp_InsertInstrumentByMusician", cn))
                        {
                            cmd.Connection = cn;
                            cmd.CommandType = CommandType.StoredProcedure;
                            cmd.Parameters.AddWithValue("@instrument_id", obj.Id);
                            cmd.Parameters.AddWithValue("@musician_id", musician.Id);

                            SqlParameter param =
                                new SqlParameter("@instrument_musician_id", SqlDbType.Int);
                            param.Direction = ParameterDirection.Output;
                            cmd.Parameters.Add(param);
                            cmd.ExecuteNonQuery();
                        }
                    }
                }
            }

            this.RaiseListChangedEvents = true;
        }

        /// <summary>
        /// Hides the public member so that Add is accessible only from within
        /// this object.
        /// </summary>
        /// <param name="instrument">The instrument.</param>
        private new void Add(Instrument instrument)
        {
            base.Add(instrument);
        }

        private void Remove(int instrumentId)
        {
            foreach (Instrument instrument in this)
            {
                if (instrument.Id == instrumentId)
                {
                    Remove(instrument);
                    break;
                }
            }
        }

        #endregion

        #region Factory Methods

        internal static MusicianInstrumentsList NewMusicianInstrumentsList()
        {
            return new MusicianInstrumentsList();
        }

        internal static MusicianInstrumentsList GetList(SafeDataReader dr, Musician musician)
        {
            _Musician = musician;
            return new MusicianInstrumentsList(dr);
        }

        private MusicianInstrumentsList()
        {
            MarkAsChild();
        }

        private MusicianInstrumentsList(SafeDataReader dr)
        {
            MarkAsChild();
            Fetch(dr);
        }

        /// <summary>
        /// Gets the musician associated with the list of
        /// <see cref="Instrument"/> objects.
        /// </summary>
        /// <value>The musician.</value>
        public static Musician Musician
        {
            get { return _Musician; }
        }

        #endregion

        #region Data Access

        // called to load data from the database
        private void Fetch(SafeDataReader dr)
        {
            this.RaiseListChangedEvents = false;
            this.Clear();
            while (dr.Read())
            {
                Instrument instrument = Instrument.GetInstrument(dr);
                this.Add(instrument);
            }

            this.RaiseListChangedEvents = true;
        }

        private void RemoveInstrumentFromMusician(Musician musician, int instrumentId)
        {
            using (SqlConnection cn = new SqlConnection(MusicDataAccess.ConnectionString))
            {
                cn.Open();
                SqlCommand cmd = new SqlCommand();
                cmd.Connection = cn;
                cmd.CommandType = CommandType.StoredProcedure;
                cmd.CommandText = "usp_DeleteInstrumentByMusician";
                cmd.Parameters.AddWithValue("@instrument_id", instrumentId);
                cmd.Parameters.AddWithValue("@musician_id", musician.Id);

                cmd.ExecuteNonQuery();
            }
        }

        #endregion
    }
}